from copy import deepcopy
from datetime import datetime, timedelta

def summonth(d):
    
    # accumulate previous months:
    # beginning with the 15th day of the month: the previous month can be summarized:
    # 2024.02.15 => 2023.12
    # 2024.02.16 => 2024.01
    
    sumUntilDay = datetime.now() - timedelta(days=46)
    sumTill = sumUntilDay.strftime("%Y%m")
    print("accumulate until month: "  + sumTill)   # 202312
    
    # iterate over days to accumulate months:
    dtmp = deepcopy(d)
    for x in dtmp['days']:
        # if date has a day:
            if len(x) > 6:
                currMonth = x[0:6]
                if currMonth <= sumTill:
                    print("CurrMonth: " + currMonth + "  day x: " + x)
                    if currMonth not in d['days']:
                            
                        d['days'][currMonth] = {}
                        d['days'][currMonth]['hits']    = 0;
                        d['days'][currMonth]['visits'] = 0;
                        d['days'][currMonth]['bytes_sent']   = 0;
                        d['days'][currMonth]['countries'] = {}
                        d['days'][currMonth]['source'] = {}
                        d['days'][currMonth]['serverResponseCode'] = {}
                        
                        d['days'][currMonth]['device_hits'] = {} 
                        d['days'][currMonth]['device_hits']['mobile']  = 0 
                        d['days'][currMonth]['device_hits']['tablet']  = 0 
                        d['days'][currMonth]['device_hits']['desktop'] = 0 
                        d['days'][currMonth]['device_hits']['bots']    = 0 
                        d['days'][currMonth]['device_hits']['others']  = 0 
                                               
                    d['days'][currMonth]['hits'] += dtmp['days'][x]['hits']
                    d['days'][currMonth]['visits'] += dtmp['days'][x]['visits']
                    d['days'][currMonth]['bytes_sent']   += int(dtmp['days'][x]['bytes_sent'])

                    if 'countries' in dtmp['days'][x]:
                        for ck,cv in dtmp['days'][x]['countries'].items():
                            print("country: " + ck + ": " + str(cv) )
                            print("countries" + str(d['days'][currMonth]['countries']))
                            if ck in d['days'][currMonth]['countries']:
                                d['days'][currMonth]['countries'][ck] += cv
                            else:
                                d['days'][currMonth]['countries'][ck] = cv;
                    
                    d['days'][currMonth]['device_hits']['mobile']  += d['days'][x]['device_hits']['mobile']
                    d['days'][currMonth]['device_hits']['tablet']  += d['days'][x]['device_hits']['tablet']
                    d['days'][currMonth]['device_hits']['bots']    += d['days'][x]['device_hits']['bots']
                    d['days'][currMonth]['device_hits']['desktop'] += d['days'][x]['device_hits']['desktop']
                    d['days'][currMonth]['device_hits']['others']  += d['days'][x]['device_hits']['others']

                    # external sources:
                    if 'source' in dtmp['days'][x]:
                        for ck,cv in dtmp['days'][x]['source'].items():
                            print("external Source: " + ck + ": " + str(cv) )

                            print("eS" + str(d['days'][currMonth]['source']))
                            if ck in d['days'][currMonth]['source']:
                                d['days'][currMonth]['source'][ck] += cv
                            else:
                                d['days'][currMonth]['source'][ck] = cv;

                    # Server Response Code:
                    for ck,cv in dtmp['days'][x]['serverResponseCode'].items():
                        print("Responsecode: " + ck + ": " + str(cv) )

                        print("eS" + str(d['days'][currMonth]['serverResponseCode']))
                        if ck in d['days'][currMonth]['serverResponseCode']:
                            d['days'][currMonth]['serverResponseCode'][ck] += cv
                        else:
                            d['days'][currMonth]['serverResponseCode'][ck] = cv;

                    del d['days'][x]
        
            # sort accumulated countries              
            for x in d['days']:
                if len(x) == 6:
                    d['days'][x]['countries'] = dict(sorted(d['days'][x]['countries'].items(), key=lambda item: item[1], reverse=True))
   
    return d



from urllib.parse import urlparse
def upd(
    d,
    i,
    visitIP):
        
    dt = i['ymd']

    # init a new day with minimal attributes:
    if dt not in d['days']:
        d['days'][dt] = {}
        d['days'][dt]['hits']    = 0;
        d['days'][dt]['visits']  = 0;
        d['days'][dt]['bytes_sent'] = 0;
        d['days'][dt]['serverResponseCode'] = {}
        d['days'][dt]['device_hits'] = {} 
        d['days'][dt]['device_hits']['mobile']  = 0 
        d['days'][dt]['device_hits']['tablet']  = 0 
        d['days'][dt]['device_hits']['desktop'] = 0 
        d['days'][dt]['device_hits']['bots']    = 0 
        d['days'][dt]['device_hits']['others']  = 0 

    # update statistics
    d['days'][dt]['hits'] = d['days'][dt]['hits'] + 1
    d['days'][dt]['bytes_sent'] = str(int(d['days'][dt]['bytes_sent']) + int(i['bytes_sent']))

    if i['status'] in d['days'][dt]['serverResponseCode']:
        d['days'][dt]['serverResponseCode'][i['status']] += 1
    else:
        d['days'][dt]['serverResponseCode'][i['status']] = 1

    # process country if available:
    if 'country' in i:
        if 'countries' not in d['days'][dt]:
            d['days'][dt]['countries'] = {}
        if i['country'] not in d['days'][dt]['countries']:
            d['days'][dt]['countries'][i['country']] = 0;
        d['days'][dt]['countries'][i['country']] = d['days'][dt]['countries'][i['country']] + 1;

    # update hits for device types:
    devCla = detectDeviceClass(i['user_agent']) 
    if devCla not in d['days'][dt]['device_hits']:
        d['days'][dt]['device_hits'][devCla] = 0 
    d['days'][dt]['device_hits'][devCla] = d['days'][dt]['device_hits'][devCla] + 1 

    # update external source:
    if 'referer' in i and i['referer'] != "-":
        if 'source' not in d['days'][dt]:
            d['days'][dt]['source'] = {}
        o = urlparse(i['referer'])
        print("host: " + o.hostname)
        if o.hostname not in d['days'][dt]['source']:
            d['days'][dt]['source'][o.hostname] = 0;
        d['days'][dt]['source'][o.hostname] += 1;
                    
    d['timelastrec'] = i['timestamp']

    # update visits
    if i['ip'] not in visitIP:
        d['days'][dt]['visits'] = d['days'][dt]['visits'] + 1;
        visitIP[i['ip']] = 1

    # update hits per url
    if i['status'] == '200':
        if 'topurl' not in d['days'][dt]:
            d['days'][dt]['topurl'] = {}
        if i['request'] not in d['days'][dt]['topurl']:
            d['days'][dt]['topurl'][i['request']] = 0;
        d['days'][dt]['topurl'][i['request']] += 1;
        
    # update quality: internal server error
    if i['status'] == '500':
        if 'quality' not in d['days'][dt]:
            d['days'][dt]['quality'] = {}
        if i['request'] not in d['days'][dt]['quality']:
            d['days'][dt]['quality'][i['request']] = {}
            d['days'][dt]['quality'][i['request']]['from'] = i['referer']
            d['days'][dt]['quality'][i['request']]['status'] = i['status']
            d['days'][dt]['quality'][i['request']]['count'] = 1
            d['days'][dt]['quality'][i['request']]['comment'] = 'internal server error'
        else:
            d['days'][dt]['quality'][i['request']]['count'] += 1

    # update quality: internal redirects    
    if len(i['referer']) > 1 and (i['status'] == '301' or i['status'] == '302'):
        if 'quality' not in d['days'][dt]:
            d['days'][dt]['quality'] = {}
        if i['request'] not in d['days'][dt]['quality']:
            d['days'][dt]['quality'][i['request']] = {}
            d['days'][dt]['quality'][i['request']]['from'] = i['referer']
            d['days'][dt]['quality'][i['request']]['status'] = i['status']
            d['days'][dt]['quality'][i['request']]['count'] = 1
            d['days'][dt]['quality'][i['request']]['comment'] = 'internal redirect'
        else:
            d['days'][dt]['quality'][i['request']]['count'] += 1

    # update quality: internal not found    
    if len(i['referer']) > 1 and (i['status'] == '404'):
        if 'quality' not in d['days'][dt]:
            d['days'][dt]['quality'] = {}
        if i['request'] not in d['days'][dt]['quality']:
            d['days'][dt]['quality'][i['request']] = {}
            d['days'][dt]['quality'][i['request']]['from'] = i['referer']
            d['days'][dt]['quality'][i['request']]['status'] = i['status']
            d['days'][dt]['quality'][i['request']]['count'] = 1
            d['days'][dt]['quality'][i['request']]['comment'] = 'internal not found'
        else:
            d['days'][dt]['quality'][i['request']]['count'] += 1

    return d, visitIP

def detectDeviceClass(ua):
  if ua is None:
     return 'others'
  if (ua.lower().find('bot') > 0):
     return 'bots'
  if (ua.lower().find('python') > 0):
     return 'bots'
  if (ua.lower().find('mediapartner') > 0):
     return 'others'
  if (ua.lower().find('curl') > 0):
     return 'others'
  if (ua.lower().find('ipad') > 0):
     return 'tablet'
  if (ua.lower().find('mobile') > 0):
     return 'mobile'
  if (ua.lower().find('android') > 0):
     return 'tablet'
  return 'desktop'



# -*- coding: utf-8 -*-
import io, sys, argparse, json, ast
from operator import itemgetter
from copy import deepcopy
from importlib.metadata import version
from datetime import datetime

def genHeader():
    h  = '<!doctype html><html lang="en"><head>'
    h += '<title>Web Statistics and Analysis</title>'
    h += '<meta charset="utf-8"><meta name="viewport" content="width=device-width, initial-scale=1">'
    h += '<meta name="robots" content="index,follow">'
    h += '<style>'
    h += '* { font-family: "\'Helvetica Neue\', \'Helvetica\', \'Arial\', sans-serif";'
    h += '    font-size: 12px; }'
    h += '</style>'
    h += '</head>' + "\n"
    h += '<body>'
    h += '<div><canvas id="myChart"></canvas></div>'
    h += '<script src="https://cdn.jsdelivr.net/npm/chart.js"></script>'
    h += '<script>' + "\n" + 'const ctx = document.getElementById(\'myChart\');' + "\n"
    return h
    
def runGenCockpit(infile, outfile):
    with open(infile) as json_file:
        d = json.load(json_file) 

        lbl = []
        dta = []
        dta_mobile = []
        dta_tablet = []
        dta_bots = []

        try:
            for e in d['days']:
                lbl.append(e)
                if 'desktop' in d['days'][e]['device_hits']:
                    dta.append(d['days'][e]['device_hits']['desktop'])
                else:
                    dta.append(0)
                if 'mobile' in d['days'][e]['device_hits']:
                    dta_mobile.append(d['days'][e]['device_hits']['mobile'])
                else:
                    dta_mobile.append(0)
                if 'tablet' in d['days'][e]['device_hits']:
                    dta_tablet.append(d['days'][e]['device_hits']['tablet'])
                else:
                    dta_tablet.append(0)
                if 'bots' in d['days'][e]['device_hits']:
                    dta_bots.append(d['days'][e]['device_hits']['bots'])
                else:
                    dta_bots.append(0)
                # add "others" to "bots" in last element of the list:
                if 'others' in d['days'][e]['device_hits']:
                    dta_bots[-1] += d['days'][e]['device_hits']['others']
  
        except KeyError:
            print('KeyError occured!' + str(d['days'][e]) )
            raise

        h = genHeader()
        h += 'new Chart(ctx, {'  + "\n"
        h += ' responsive: true' + "\n"
        h += ' ,options: { scales: {x:{ stacked: true}, y:{ stacked: true } }' + "\n"
        h += ' ,plugins: { subtitle: { display: true, text: \'Hits per Device Class as of ' + d['timelastrec'][0:8] + ' ' + d['timelastrec'][7:6] + '\'} }' + "\n"
        h +=  ' },' + "\n"
        h += ' data: { ' + "\n" 
        h += '   datasets: [' + "\n"
        h += '      { type: \'line\',label: \'bots and others\',data: ' + str(dta_bots) + '}' + "\n"
        h += '     ,{ type: \'bar\', label: \'Desktop\', data: ' + str(dta) + ',backgroundColor: \'#42c5f5\'}' + "\n"
        h += '     ,{ type: \'bar\', label: \'Mobile\',  data: ' + str(dta_mobile) + ',backgroundColor: \'#42f5aa\'}' + "\n"
        h += '     ,{ type: \'bar\', label: \'Tablets\', data: ' + str(dta_tablet) + ',backgroundColor: \'#f5a742\'}' + "\n"
        h += '    ],' + "\n"
        h += '    labels: ' + str(lbl) + "\n"
        h += ' },' + "\n" + '});' + "\n"
        h += '</script>' + "\n"

        lastDate = list(d['days'].keys())[-1]
        actYearMonth = lastDate[0:6]
        
        tquality = {}   # nested dictionary!
        for k, v in sorted(d['days'].items(), key=itemgetter(0), reverse=True):
            curYearMonth = k[0:6]
            if curYearMonth == actYearMonth:
                if 'quality' in d['days'][k]:
                    for sk,sv in d['days'][k]['quality'].items():
                        print('sk: ' + str(sk) + ' sv: ' + str(sv))
                        if sk not in tquality:
                            tquality[sk] = {}
                            tquality[sk]['count'] = 1
                            tquality[sk]['status'] = sv['status']
                            tquality[sk]['from'] = sv['from']
                            tquality[sk]['comment'] = sv['comment']
                        else:
                            tquality[sk]['count']  += 1

        if len(tquality) > 0:
            h += '<h2>Quality Improvements</h2>'
            h += '<table><thead><tr><th scope="col">affected URL</th><th scope="col">Status</th><th scope="col">affected URL is called by</th><th scope="col">Count</th><th scope="col">Remark</th></tr></thead>'
            i = 0
            for k, v in sorted(tquality.items(), key=lambda x: (x[1]['count']), reverse=True):
                h += '<tr><td>' + str(k) + "</td><td>" + str(v['status']) + "</td>"
                h += '<td>' + v['from'] + "</td>"
                h += '<td>' + str(v['count']) + "</td>"
                h += '<td>' + v['comment'] + "</td></tr>"
                i += 1
                if i == 5:
                    break
            h += '</table>' + "\n"

        # Top Sources
        tsource = {}
        for y in d['days']:
            curYearMonth = y[0:6]
            if curYearMonth == actYearMonth:
               for sk,sv in d['days'][y]['source'].items():
                   print('sk: ' + str(sk) + ' sv: ' + str(sv))
                   if sk not in tsource:
                       tsource[sk] = 0
                   tsource[sk] += sv

        h += '<h2>Top 5 Sources</h2>'
        h += '<table>'
        h += '<thead><tr><th scope="col">Source Domain</th><th scope="col">Source Hits</th></tr></thead>'
        i = 0
        for k, v in sorted(tsource.items(), key=itemgetter(1), reverse=True):
             h += '<tr><td>' + str(k) + "</td><td>" + str(v) + "</td></tr>"
             i += 1
             if i == 5:
                 break
        h += '</table>' + "\n"

        #for k, v in reversed(d['days'].items()):
        #    if 'source' in d['days'][k]:
        #        for sk, sv in d['days'][k]['source'].items():

        # Top Countries
        tcountries = {}
        for y in d['days']:
            curYearMonth = y[0:6]
            print (curYearMonth + " " + actYearMonth)
            if curYearMonth == actYearMonth:
               for co,cv in d['days'][y]['countries'].items():
                   if co not in tcountries:
                       tcountries[co] = 0
                   tcountries[co] += cv

        if len(tcountries) > 0:
            h += '<h2>Top 5 Countries</h2>' + "\n"
            h += '<table>'
            h += '<thead><tr><th scope="col">Source Country</th><th scope="col">Source Hits</th></tr></thead>'
            i = 0
            for k, v in sorted(tcountries.items(), key=itemgetter(1), reverse=True):
                h += '<tr><td>' + str(k) + "</td><td>" + str(v) + "</td></tr>"
                i += 1
                if i == 5:
                    break
            h += '</table>' + "\n"
        
        h += '</body></html>'

        # write html to file:
        outfile = open(outfile, "w")
        outfile.write(h)
        outfile.close()

        return 0


if __name__ == "__main__":
    parser = argparse.ArgumentParser(allow_abbrev=False,
        prog='a9x_webstatistics',
        epilog="Version: "+ version('a9x-webstatistics')
    )
    parser.add_argument("-i", "--infile", help="json file that contains calculated statistics", default="webstat.json")
    parser.add_argument("-o", "--outfile", help="html file that contains html cockpit", default="webstat.html")
    args, unknown = parser.parse_known_args()

    runGenCockpit(infile=args.infile, outfile=args.outfile)
